@@ -41,18 +41,22 @@ import platform
 import numpy
 import scipy
 import inspect
+from qutip.utilities import _blas_info
 import qutip.settings
+from qutip.hardware_info import hardware_info
 
 
 def about():
     """
-    About box for qutip. Gives version numbers for
+    About box for QuTiP. Gives version numbers for
     QuTiP, NumPy, SciPy, Cython, and MatPlotLib.
     """
     print("")
     print("QuTiP: Quantum Toolbox in Python")
     print("Copyright (c) 2011 and later.")
-    print("Paul D. Nation & Robert J. Johansson")
+    print(("A. J. Pitchford, P. D. Nation, "
+            "R. J. Johansson, A. Grimsmo, "
+            "and C. Granade"))
     print("")
     print("QuTiP Version:      %s" % qutip.__version__)
     print("Numpy Version:      %s" % numpy.__version__)
@@ -69,9 +73,11 @@ def about():
     except:
         matplotlib_ver = 'None'
     print("Matplotlib Version: %s" % matplotlib_ver)
-    print("Fortran mcsolver:   %s" % str(qutip.settings.fortran))
-    print("scikits.umfpack:    %s" % str(qutip.settings.umfpack))
     print("Python Version:     %d.%d.%d" % sys.version_info[0:3])
+    print("Number of CPUs:     %s" % hardware_info()['cpus'])
+    print("BLAS Info:          %s" % _blas_info())
+    print("OPENMP Installed:   %s" % str(qutip.settings.has_openmp))
+    print("INTEL MKL Ext:      %s" % str(qutip.settings.has_mkl))
     print("Platform Info:      %s (%s)" % (platform.system(),
                                            platform.machine()))
     qutip_install_path = os.path.dirname(inspect.getsourcefile(qutip))
@@ -80,3 +86,5 @@ def about():
 
 if __name__ == "__main__":
     about()
+
+
