@@ -53,12 +53,11 @@ from qutip.states import ket2dm
 from qutip.states import projection
 from qutip.solver import Options
 from qutip.propagator import propagator
-from qutip.solver import Result
+from qutip.solver import Result, _solver_safety_check
 from qutip.cy.spmatfuncs import cy_ode_rhs
 from qutip.expect import expect
 from qutip.utilities import n_thermal
 
-
 def floquet_modes(H, T, args=None, sort=False, U=None):
     """
     Calculate the initial Floquet modes Phi_alpha(0) for a driven system with
@@ -126,7 +125,6 @@ def floquet_modes(H, T, args=None, sort=False, U=None):
 
     return kets_order, e_quasi[order]
 
-
 def floquet_modes_t(f_modes_0, f_energies, t, H, T, args=None):
     """
     Calculate the Floquet modes at times tlist Phi_alpha(tlist) propagting the
@@ -179,7 +177,6 @@ def floquet_modes_t(f_modes_0, f_energies, t, H, T, args=None):
 
     return f_modes_t
 
-
 def floquet_modes_table(f_modes_0, f_energies, tlist, H, T, args=None):
     """
     Pre-calculate the Floquet modes for a range of times spanning the floquet
@@ -232,7 +229,6 @@ def floquet_modes_table(f_modes_0, f_energies, tlist, H, T, args=None):
 
     return f_modes_table_t
 
-
 def floquet_modes_t_lookup(f_modes_table_t, t, T):
     """
     Lookup the floquet mode at time t in the pre-calculated table of floquet
@@ -274,7 +270,6 @@ def floquet_modes_t_lookup(f_modes_table_t, t, T):
 
     return f_modes_table_t[t_idx]
 
-
 def floquet_states(f_modes_t, f_energies, t):
     """
     Evaluate the floquet states at time t given the Floquet modes at that time.
@@ -303,7 +298,6 @@ def floquet_states(f_modes_t, f_energies, t):
     return [(f_modes_t[i] * exp(-1j * f_energies[i] * t))
             for i in np.arange(len(f_energies))]
 
-
 def floquet_states_t(f_modes_0, f_energies, t, H, T, args=None):
     """
     Evaluate the floquet states at time t given the initial Floquet modes.
@@ -342,7 +336,6 @@ def floquet_states_t(f_modes_0, f_energies, t, H, T, args=None):
     return [(f_modes_t[i] * exp(-1j * f_energies[i] * t))
             for i in np.arange(len(f_energies))]
 
-
 def floquet_wavefunction(f_modes_t, f_energies, f_coeff, t):
     """
     Evaluate the wavefunction for a time t using the Floquet state
@@ -374,7 +367,6 @@ def floquet_wavefunction(f_modes_t, f_energies, f_coeff, t):
     return sum([f_modes_t[i] * exp(-1j * f_energies[i] * t) * f_coeff[i]
                 for i in np.arange(len(f_energies))])
 
-
 def floquet_wavefunction_t(f_modes_0, f_energies, f_coeff, t, H, T, args=None):
     """
     Evaluate the wavefunction for a time t using the Floquet state
@@ -417,7 +409,6 @@ def floquet_wavefunction_t(f_modes_0, f_energies, f_coeff, t, H, T, args=None):
     return sum([f_states_t[i] * f_coeff[i]
                 for i in np.arange(len(f_energies))])
 
-
 def floquet_state_decomposition(f_states, f_energies, psi):
     """
     Decompose the wavefunction `psi` (typically an initial state) in terms of
@@ -446,7 +437,6 @@ def floquet_state_decomposition(f_states, f_energies, psi):
     return [(f_states[i].dag() * psi).data[0, 0]
             for i in np.arange(len(f_energies))]
 
-
 def fsesolve(H, psi0, tlist, e_ops=[], T=None, args={}, Tsteps=100):
     """
     Solve the Schrodinger equation using the Floquet formalism.
@@ -547,7 +537,6 @@ def fsesolve(H, psi0, tlist, e_ops=[], T=None, args={}, Tsteps=100):
 
     return output
 
-
 def floquet_master_equation_rates(f_modes_0, f_energies, c_op, H, T,
                                   args, J_cb, w_th, kmax=5,
                                   f_modes_table_t=None):
@@ -655,7 +644,6 @@ def floquet_master_equation_rates(f_modes_0, f_energies, c_op, H, T,
 
     return Delta, X, Gamma, A
 
-
 def floquet_collapse_operators(A):
     """
     Construct collapse operators corresponding to the Floquet-Markov
@@ -683,7 +671,6 @@ def floquet_collapse_operators(A):
 
     return c_ops
 
-
 def floquet_master_equation_tensor(Alist, f_energies):
     """
     Construct a tensor that represents the master equation in the floquet
@@ -718,17 +705,14 @@ def floquet_master_equation_tensor(Alist, f_energies):
         Alist = [Alist]
         N, M = np.shape(Alist[0])
 
-    R = Qobj(scipy.sparse.csr_matrix((N * N, N * N)), [[N, N], [N, N]],
-             [N * N, N * N])
-
-    R.data = R.data.tolil()
+    Rdata_lil = scipy.sparse.lil_matrix((N * N, N * N), dtype=complex)
     for I in range(N * N):
         a, b = vec2mat_index(N, I)
         for J in range(N * N):
             c, d = vec2mat_index(N, J)
 
-            R.data[I, J] = -1.0j * (f_energies[a] - f_energies[b]) * \
-                (a == c) * (b == d)
+            R = -1.0j * (f_energies[a] - f_energies[b])*(a == c)*(b == d)
+            Rdata_lil[I, J] = R
 
             for A in Alist:
                 s1 = s2 = 0
@@ -740,11 +724,9 @@ def floquet_master_equation_tensor(Alist, f_energies):
                 dR = (a == b) * s1 - 0.5 * (1 - (a == b)) * s2
 
                 if dR != 0.0:
-                    R.data[I, J] += dR
-
-    R.data = R.data.tocsr()
-    return R
+                    Rdata_lil[I, J] += dR
 
+    return Qobj(Rdata_lil, [[N, N], [N, N]], [N*N, N*N])
 
 def floquet_master_equation_steadystate(H, A):
     """
@@ -755,7 +737,6 @@ def floquet_master_equation_steadystate(H, A):
     rho_ss = steadystate(H, c_ops)
     return rho_ss
 
-
 def floquet_basis_transform(f_modes, f_energies, rho0):
     """
     Make a basis transform that takes rho0 from the floquet basis to the
@@ -763,7 +744,6 @@ def floquet_basis_transform(f_modes, f_energies, rho0):
     """
     return rho0.transform(f_modes, True)
 
-
 # -----------------------------------------------------------------------------
 # Floquet-Markov master equation
 #
@@ -853,7 +833,7 @@ def floquet_markov_mesolve(R, ekets, rho0, tlist, e_ops, f_modes_table=None,
         if not r.successful():
             break
 
-        rho.data = vec2mat(r.y)
+        rho = Qobj(vec2mat(r.y), rho0.dims, rho0.shape)
 
         if expt_callback:
             # use callback method
@@ -885,13 +865,13 @@ def floquet_markov_mesolve(R, ekets, rho0, tlist, e_ops, f_modes_table=None,
 
     return output
 
-
 # -----------------------------------------------------------------------------
 # Solve the Floquet-Markov master equation
 #
 #
-def fmmesolve(H, rho0, tlist, c_ops, e_ops=[], spectra_cb=[], T=None,
-              args={}, options=Options(), floquet_basis=True, kmax=5):
+def fmmesolve(H, rho0, tlist, c_ops=[], e_ops=[], spectra_cb=[], T=None,
+              args={}, options=Options(), floquet_basis=True, kmax=5,
+              _safe_mode=True):
     """
     Solve the dynamics for the system using the Floquet-Markov master equation.
 
@@ -954,7 +934,10 @@ def fmmesolve(H, rho0, tlist, c_ops, e_ops=[], spectra_cb=[], T=None,
         An instance of the class :class:`qutip.solver`, which contains either
         an *array* of expectation values for the times specified by `tlist`.
     """
-
+    
+    if _safe_mode:
+        _solver_safety_check(H, rho0, c_ops, e_ops, args)
+    
     if T is None:
         T = max(tlist)
 
