@@ -37,20 +37,26 @@ import types
 import numpy as np
 import scipy.linalg as la
 import functools
-
+import scipy.sparse as sp
 from qutip.qobj import Qobj
-from qutip.rhs_generate import rhs_clear
+from qutip.tensor import tensor
+from qutip.operators import qeye
+from qutip.rhs_generate import (rhs_generate, rhs_clear, _td_format_check)
 from qutip.superoperator import (vec2mat, mat2vec,
                                  vector_to_operator, operator_to_vector)
+from qutip.sparse import sp_reshape
+from qutip.cy.sparse_utils import unit_row_norm
 from qutip.mesolve import mesolve
 from qutip.sesolve import sesolve
 from qutip.states import basis
-from qutip.solver import Options
+from qutip.solver import Options, _solver_safety_check, config
+from qutip.parallel import parallel_map, _default_kwargs
 from qutip.ui.progressbar import BaseProgressBar, TextProgressBar
 
 
-def propagator(H, t, c_op_list, args=None, options=None, sparse=False,
-               progress_bar=None):
+def propagator(H, t, c_op_list=[], args={}, options=None,
+               unitary_mode='batch', parallel=False, 
+               progress_bar=None, **kwargs):
     """
     Calculate the propagator U(t) for the density matrix or wave function such
     that :math:`\psi(t) = U(t)\psi(0)` or
@@ -78,6 +84,14 @@ def propagator(H, t, c_op_list, args=None, options=None, sparse=False,
     options : :class:`qutip.Options`
         with options for the ODE solver.
 
+    unitary_mode = str ('batch', 'single')
+        Solve all basis vectors simulaneously ('batch') or individually 
+        ('single').
+    
+    parallel : bool {False, True}
+        Run the propagator in parallel mode. This will override the 
+        unitary_mode settings if set to True.
+    
     progress_bar: BaseProgressBar
         Optional instance of BaseProgressBar, or a subclass thereof, for
         showing the progress of the simulation. By default no progress bar
@@ -89,7 +103,12 @@ def propagator(H, t, c_op_list, args=None, options=None, sparse=False,
         Instance representing the propagator :math:`U(t)`.
 
     """
-
+    kw = _default_kwargs()
+    if 'num_cpus' in kwargs:
+        num_cpus = kwargs['num_cpus']
+    else:
+        num_cpus = kw['num_cpus']
+    
     if progress_bar is None:
         progress_bar = BaseProgressBar()
     elif progress_bar is True:
@@ -105,6 +124,8 @@ def propagator(H, t, c_op_list, args=None, options=None, sparse=False,
     else:
         tlist = t
 
+    td_type = _td_format_check(H, c_op_list, solver='me')
+        
     if isinstance(H, (types.FunctionType, types.BuiltinFunctionType,
                       functools.partial)):
         H0 = H(0.0, args)
@@ -112,85 +133,138 @@ def propagator(H, t, c_op_list, args=None, options=None, sparse=False,
         H0 = H[0][0] if isinstance(H[0], list) else H[0]
     else:
         H0 = H
-
+    
     if len(c_op_list) == 0 and H0.isoper:
         # calculate propagator for the wave function
 
         N = H0.shape[0]
         dims = H0.dims
-        u = np.zeros([N, N, len(tlist)], dtype=complex)
+        
+        if parallel:
+            unitary_mode = 'single'
+            u = np.zeros([N, N, len(tlist)], dtype=complex)
+            output = parallel_map(_parallel_sesolve, range(N),
+                                  task_args=(N, H, tlist, args, options),
+                                  progress_bar=progress_bar, num_cpus=num_cpus)
+            for n in range(N):
+                for k, t in enumerate(tlist):
+                    u[:, n, k] = output[n].states[k].full().T 
+        else:
+            if unitary_mode == 'single':
+                u = np.zeros([N, N, len(tlist)], dtype=complex)
+                progress_bar.start(N)
+                for n in range(0, N):
+                    progress_bar.update(n)
+                    psi0 = basis(N, n)
+                    output = sesolve(H, psi0, tlist, [], args, options,
+                                     _safe_mode=False)
+                    for k, t in enumerate(tlist):
+                        u[:, n, k] = output.states[k].full().T
+                    progress_bar.finished()
 
-        progress_bar.start(N)
-        for n in range(0, N):
-            progress_bar.update(n)
-            psi0 = basis(N, n)
-            output = sesolve(H, psi0, tlist, [], args, options)
-            for k, t in enumerate(tlist):
-                u[:, n, k] = output.states[k].full().T
-        progress_bar.finished()
 
-        # todo: evolving a batch of wave functions:
-        # psi_0_list = [basis(N, n) for n in range(N)]
-        # psi_t_list = mesolve(H, psi_0_list, [0, t], [], [], args, options)
-        # for n in range(0, N):
-        #    u[:,n] = psi_t_list[n][1].full().T
+            elif unitary_mode =='batch':
+                u = np.zeros(len(tlist), dtype=object)
+                _rows = np.array([(N+1)*m for m in range(N)])
+                _cols = np.zeros_like(_rows)
+                _data = np.ones_like(_rows, dtype=complex)
+                psi0 = Qobj(sp.coo_matrix((_data, (_rows, _cols))).tocsr())
+                if td_type[1] > 0 or td_type[2] > 0:
+                    H2 = []
+                    for k in range(len(H)):
+                        if isinstance(H[k], list):
+                            H2.append([tensor(qeye(N), H[k][0]), H[k][1]])
+                        else:
+                            H2.append(tensor(qeye(N), H[k]))
+                else:
+                    H2 = tensor(qeye(N), H)
+                output = sesolve(H2, psi0, tlist, [],
+                                 args=args, _safe_mode=False,
+                                 options=Options(normalize_output=False))
+                for k, t in enumerate(tlist):
+                    u[k] = sp_reshape(output.states[k].data, (N, N))
+                    unit_row_norm(u[k].data, u[k].indptr, u[k].shape[0])
+                    u[k] = u[k].T.tocsr()
+
+            else:
+                raise Exception('Invalid unitary mode.')
+                        
 
     elif len(c_op_list) == 0 and H0.issuper:
         # calculate the propagator for the vector representation of the
         # density matrix (a superoperator propagator)
-
+        unitary_mode = 'single'
         N = H0.shape[0]
+        sqrt_N = int(np.sqrt(N))
         dims = H0.dims
-
+        
         u = np.zeros([N, N, len(tlist)], dtype=complex)
 
-        progress_bar.start(N)
-        for n in range(0, N):
-            progress_bar.update(n)
-            psi0 = basis(N, n)
-            rho0 = Qobj(vec2mat(psi0.full()))
-            output = mesolve(H, rho0, tlist, [], [], args, options)
-            for k, t in enumerate(tlist):
-                u[:, n, k] = mat2vec(output.states[k].full()).T
-        progress_bar.finished()
+        if parallel:
+            output = parallel_map(_parallel_mesolve,range(N * N),
+                                  task_args=(
+                                      sqrt_N, H, tlist, c_op_list, args,
+                                      options),
+                                  progress_bar=progress_bar, num_cpus=num_cpus)
+            for n in range(N * N):
+                for k, t in enumerate(tlist):
+                    u[:, n, k] = mat2vec(output[n].states[k].full()).T
+        else:
+            progress_bar.start(N)
+            for n in range(0, N):
+                progress_bar.update(n)
+                col_idx, row_idx = np.unravel_index(n, (sqrt_N, sqrt_N))
+                rho0 = Qobj(sp.csr_matrix(([1], ([row_idx], [col_idx])),
+                                          shape=(sqrt_N,sqrt_N), dtype=complex)
+                            )
+                output = mesolve(H, rho0, tlist, [], [], args, options,
+                                 _safe_mode=False)
+                for k, t in enumerate(tlist):
+                    u[:, n, k] = mat2vec(output.states[k].full()).T
+            progress_bar.finished()
 
     else:
         # calculate the propagator for the vector representation of the
         # density matrix (a superoperator propagator)
-
+        unitary_mode = 'single'
         N = H0.shape[0]
         dims = [H0.dims, H0.dims]
 
         u = np.zeros([N * N, N * N, len(tlist)], dtype=complex)
-
-        if sparse:
-            progress_bar.start(N * N)
+        
+        if parallel:
+            output = parallel_map(_parallel_mesolve, range(N * N),
+                                  task_args=(
+                                      N, H, tlist, c_op_list, args, options),
+                                  progress_bar=progress_bar, num_cpus=num_cpus)
             for n in range(N * N):
-                progress_bar.update(n)
-                psi0 = basis(N * N, n)
-                psi0.dims = [dims[0], 1]
-                rho0 = vector_to_operator(psi0)
-                output = mesolve(H, rho0, tlist, c_op_list, [], args, options)
                 for k, t in enumerate(tlist):
-                    u[:, n, k] = operator_to_vector(
-                        output.states[k]).full(squeeze=True)
-            progress_bar.finished()
-
+                    u[:, n, k] = mat2vec(output[n].states[k].full()).T
         else:
             progress_bar.start(N * N)
             for n in range(N * N):
                 progress_bar.update(n)
-                psi0 = basis(N * N, n)
-                rho0 = Qobj(vec2mat(psi0.full()))
-                output = mesolve(H, rho0, tlist, c_op_list, [], args, options)
+                col_idx, row_idx = np.unravel_index(n, (N, N))
+                rho0 = Qobj(sp.csr_matrix(([1], ([row_idx], [col_idx])),
+                                          shape=(N,N), dtype=complex))
+                output = mesolve(H, rho0, tlist, c_op_list, [], args, options,
+                                 _safe_mode=False)
                 for k, t in enumerate(tlist):
                     u[:, n, k] = mat2vec(output.states[k].full()).T
             progress_bar.finished()
 
     if len(tlist) == 2:
-        return Qobj(u[:, :, 1], dims=dims)
+        if unitary_mode == 'batch':
+            return Qobj(u[-1], dims=dims)
+        else:
+            return Qobj(u[:, :, 1], dims=dims)
     else:
-        return [Qobj(u[:, :, k], dims=dims) for k in range(len(tlist))]
+        if unitary_mode == 'batch':
+            return np.array([Qobj(u[k], dims=dims)
+                             for k in range(len(tlist))], dtype=object)
+        else:
+            return np.array([Qobj(u[:, :, k], dims=dims)
+                             for k in range(len(tlist))], dtype=object)
 
 
 def _get_min_and_index(lst):
@@ -221,11 +295,28 @@ def propagator_steadystate(U):
     """
 
     evals, evecs = la.eig(U.full())
-
-    ev_min, ev_idx = _get_min_and_index(abs(evals - 1.0))
-
+    
+    shifted_vals = np.abs(evals - 1.0)
+    ev_idx = np.argmin(shifted_vals)
+    ev_min = shifted_vals[ev_idx]
     evecs = evecs.T
     rho = Qobj(vec2mat(evecs[ev_idx]), dims=U.dims[0])
     rho = rho * (1.0 / rho.tr())
     rho = 0.5 * (rho + rho.dag())  # make sure rho is herm
+    rho.isherm = True
     return rho
+
+
+def _parallel_sesolve(n, N, H, tlist, args, options):
+    psi0 = basis(N, n)
+    output = sesolve(H, psi0, tlist, [], args, options, _safe_mode=False)
+    return output
+
+def _parallel_mesolve(n, N, H, tlist, c_op_list, args, options):
+    col_idx, row_idx = np.unravel_index(n, (N, N))
+    rho0 = Qobj(sp.csr_matrix(([1], ([row_idx], [col_idx])),
+                              shape=(N,N), dtype=complex))
+    output = mesolve(H, rho0, tlist, c_op_list, [], args, options,
+                     _safe_mode=False)
+    return output
+
