@@ -32,24 +32,29 @@
 ###############################################################################
 from __future__ import division
 
+import numpy as np
+import scipy.sparse as sp
+from qutip.qobj import Qobj
+from qutip.operators import identity, qeye, sigmax
+from qutip.tensor import tensor
+from qutip.states import fock_dm
+
+from itertools import product
+from functools import partial, reduce
+from operator import mul
+
 __all__ = ['rx', 'ry', 'rz', 'sqrtnot', 'snot', 'phasegate', 'cphase', 'cnot',
            'csign', 'berkeley', 'swapalpha', 'swap', 'iswap', 'sqrtswap',
            'sqrtiswap', 'fredkin', 'toffoli', 'rotation', 'controlled_gate',
            'globalphase', 'hadamard_transform', 'gate_sequence_product',
-           'gate_expand_1toN', 'gate_expand_2toN', 'gate_expand_3toN']
-
-import numpy as np
-import scipy.sparse as sp
-from qutip.qobj import Qobj
-from qutip.operators import identity
-from qutip.tensor import tensor
-from qutip.states import fock_dm
-
+           'gate_expand_1toN', 'gate_expand_2toN', 'gate_expand_3toN',
+           'qubit_clifford_group']
 
 #
 # Single Qubit Gates
 #
 
+
 def rx(phi, N=None, target=0):
     """Single-qubit rotation for operator sigmax with angle phi.
 
@@ -177,7 +182,7 @@ shape = [2, 2], type = oper, isHerm = False
 
 def cphase(theta, N=2, control=0, target=1):
     """
-    Returns quantum object representing the phase shift gate.
+    Returns quantum object representing the controlled phase shift gate.
 
     Parameters
     ----------
@@ -468,6 +473,7 @@ shape = [4, 4], type = oper, isHerm = False
        0.70710678+0.j          0.00000000+0.j]
      [ 0.00000000+0.j   0.00000000+0.j   \
        0.00000000+0.j          1.00000000+0.j]]
+
     """
     if targets != [0, 1] and N is None:
         N = 2
@@ -582,6 +588,7 @@ def rotation(op, phi, N=None, target=0):
     -------
     result : qobj
         Quantum object for operator describing the rotation.
+
     """
     if N is not None:
         return gate_expand_1toN(rotation(op, phi), N, target)
@@ -615,6 +622,7 @@ def controlled_gate(U, N=2, control=0, target=1, control_value=1):
     -------
     result : qobj
         Quantum object representing the controlled-U gate.
+
     """
 
     if [N, control, target] == [2, 0, 1]:
@@ -649,8 +657,8 @@ shape = [2, 2], type = oper, isHerm = False
      [ 0.00000000+0.j          0.70710678+0.70710678j]]
 
     """
-    data = (np.exp(1.0j * theta)
-            * sp.eye(2 ** N, 2 ** N, dtype=complex, format="csr"))
+    data = (np.exp(1.0j * theta) * sp.eye(2 ** N, 2 ** N,
+                                          dtype=complex, format="csr"))
     return Qobj(data, dims=[[2] * N, [2] * N])
 
 
@@ -677,6 +685,7 @@ def hadamard_transform(N=1):
     -------
     q : qobj
         Quantum object representation of the N-qubit Hadamard gate.
+
     """
     data = 2 ** (-N / 2) * np.array([[(-1) ** _hamming_distance(i & j)
                                       for i in range(2 ** N)]
@@ -694,12 +703,12 @@ def gate_sequence_product(U_list, left_to_right=True):
     U_list : list
         List of gates implementing the quantum circuit.
 
-    left_to_right: Boolean
+    left_to_right : Boolean
         Check if multiplication is to be done from left to right.
 
     Returns
     -------
-    U_overall: qobj
+    U_overall : qobj
         Overall unitary matrix of a given quantum circuit.
 
     """
@@ -713,10 +722,79 @@ def gate_sequence_product(U_list, left_to_right=True):
     return U_overall
 
 
+def _powers(op, N):
+    """
+    Generator that yields powers of an operator `op`,
+    through to `N`.
+    """
+    acc = qeye(op.dims[0])
+    yield acc
+
+    for _ in range(N - 1):
+        acc *= op
+        yield acc
+
+
+def qubit_clifford_group(N=None, target=0):
+    """
+    Generates the Clifford group on a single qubit,
+    using the presentation of the group given by Ross and Selinger
+    (http://www.mathstat.dal.ca/~selinger/newsynth/).
+
+    Parameters
+    -----------
+
+    N : int or None
+        Number of qubits on which each operator is to be defined
+        (default: 1).
+    target : int
+        Index of the target qubit on which the single-qubit
+        Clifford operators are to act.
+
+    Yields
+    ------
+
+    op : Qobj
+        Clifford operators, represented as Qobj instances.
+
+    """
+
+    # The Ross-Selinger presentation of the single-qubit Clifford
+    # group expresses each element in the form C_{ijk} = E^i X^j S^k
+    # for gates E, X and S, and for i in range(3), j in range(2) and
+    # k in range(4).
+    #
+    # We start by defining these gates. E is defined in terms of H,
+    # \omega and S, so we define \omega and H first.
+    w = np.exp(1j * 2 * np.pi / 8)
+    H = snot()
+
+    X = sigmax()
+    S = phasegate(np.pi / 2)
+    E = H * (S ** 3) * w ** 3
+
+    for op in map(partial(reduce, mul), product(_powers(E, 3),
+                  _powers(X, 2), _powers(S, 4))):
+
+        # partial(reduce, mul) returns a function that takes products
+        # of its argument, by analogy to sum. Note that by analogy,
+        # sum can be written as partial(reduce, add).
+
+        # product(...) yields the Cartesian product of its arguments.
+        # Here, each element is a tuple (E**i, X**j, S**k) such that
+        # partial(reduce, mul) acting on the tuple yields E**i * X**j * S**k.
+
+        # Finally, we optionally expand the gate.
+        if N is not None:
+            yield gate_expand_1toN(op, N, target)
+        else:
+            yield op
+
 #
 # Gate Expand
 #
 
+
 def gate_expand_1toN(U, N, target):
     """
     Create a Qobj representing a one-qubit gate that act on a system with N
@@ -737,6 +815,7 @@ def gate_expand_1toN(U, N, target):
     -------
     gate : qobj
         Quantum object representation of N-qubit gate.
+
     """
 
     if N < 1:
@@ -775,6 +854,7 @@ def gate_expand_2toN(U, N, control=None, target=None, targets=None):
     -------
     gate : qobj
         Quantum object representation of N-qubit gate.
+
     """
 
     if targets is not None:
@@ -831,19 +911,22 @@ def gate_expand_3toN(U, N, controls=[0, 1], target=2):
     -------
     gate : qobj
         Quantum object representation of N-qubit gate.
+
     """
 
     if N < 3:
         raise ValueError("integer N must be larger or equal to 3")
 
     if controls[0] >= N or controls[1] >= N or target >= N:
-        raise ValueError(
-            "control and not target is None must be integer < integer N")
+        raise ValueError("control and not target is None."
+                         " Must be integer < integer N")
 
-    if (controls[0] == target or controls[1] == target
-            or controls[0] == controls[1]):
-        raise ValueError(
-            "controls[0], controls[1], and target cannot be equal")
+    if (controls[0] == target or
+        controls[1] == target or
+        controls[0] == controls[1]):
+
+        raise ValueError("controls[0], controls[1], and target"
+                         " cannot be equal")
 
     p = list(range(N))
     p1 = list(range(N))
