@@ -47,8 +47,12 @@ auto_tidyup_atol = 1e-12
 num_cpus = 0
 # flag indicating if fortran module is installed
 fortran = False
-# flag indicating if scikits.umfpack is installed
-umfpack = False
+# path to the MKL library
+mkl_lib = None
+# Flag if mkl_lib is found
+has_mkl = False
+# Has OPENMP
+has_openmp = False
 # debug mode for development
 debug = False
 # are we in IPython? Note that this cannot be
@@ -63,74 +67,18 @@ log_handler = 'default'
 # Allow for a colorblind mode that uses different colormaps
 # and plotting options by default.
 colorblind_safe = False
-
+# Sets the threshold for matrix NNZ where OPENMP
+# turns on. This is automatically calculated and 
+# put in the qutiprc file.  This value is here in case
+# that failts
+openmp_thresh = 10000
 # Note that since logging depends on settings,
 # if we want to do any logging here, it must be manually
 # configured, rather than through _logging.get_logger().
-import logging
-_logger = logging.getLogger(__name__)
-_logger.addHandler(logging.NullHandler())
-del logging  # Don't leak names!
-
-
-def load_rc_file(rc_file):
-    """
-    Load settings for the qutip RC file, by default .qutiprc in the user's home
-    directory.
-    """
-    global auto_tidyup, auto_herm, auto_tidyup_atol, num_cpus, debug, atol
-    global log_handler, colorblind_safe
-
-    # Try to pull in configobj to do nicer handling of
-    # config files instead of doing manual parsing.
-    # We pull it in here to avoid throwing an error if configobj is missing.
-    try:
-        import configobj as _cobj
-        import pkg_resources
-        import validate
-    except ImportError:
-        # Don't bother warning unless the rc_file exists.
-        import os
-        if os.path.exists(rc_file):
-            _logger.warn("configobj missing, not loading rc_file.", exc_info=1)
-        return
-
-    # Try to find the specification for the config file, then
-    # use it to load the actual config.
-    config = _cobj.ConfigObj(
-        rc_file,
-        configspec=pkg_resources.resource_filename('qutip', 'configspec.ini'),
-        # doesn't throw an error if qutiprc is missing.
-        file_error=False
-    )
-
-    # Next, validate the loaded config file against the specs.
-    validator = validate.Validator()
-    result = config.validate(validator)
-
-    # configobj's validator returns the literal True if everything
-    # worked, and returns a dictionary of which keys fails otherwise.
-    # This motivates a very un-Pythonic way of checking for results,
-    # but it's the configobj idiom.
-    if result is not True:
-        # OK, find which keys are bad.
-        bad_keys = {key for key, val in result.iteritems() if not val}
-        _logger.warn('Invalid configuration options in {}: {}'.format(
-            rc_file, bad_keys
-        ))
-    else:
-        bad_keys = {}
-
-    # Now that everything's been validated, we apply the config
-    # file to the global settings.
-    for config_key in (
-        'auto_tidyup', 'auto_herm', 'atol', 'auto_tidyup_atol',
-        'num_cpus', 'debug', 'log_handler', 'colorblind_safe'
-    ):
-        if config_key in config and config_key not in bad_keys:
-            _logger.debug(
-                "Applying configuration setting {} = {}.".format(
-                    config_key, config[config_key]
-                )
-            )
-            globals()[config_key] = config[config_key]
+try:
+    import logging
+    _logger = logging.getLogger(__name__)
+    _logger.addHandler(logging.NullHandler())
+    del logging  # Don't leak names!
+except:
+    _logger = None
