@@ -48,6 +48,8 @@ from qutip.qobj import Qobj
 from qutip.operators import destroy, jmat
 from qutip.tensor import tensor
 
+from qutip.fastsparse import fast_csr_matrix
+
 
 def basis(N, n=0, offset=0):
     """Generates the vector representation of a Fock state.
@@ -102,11 +104,11 @@ def basis(N, n=0, offset=0):
     if n - offset > (N - 1):  # check if n is within bounds
         raise ValueError("basis vector index need to be in n <= N-1")
 
-    bas = sp.lil_matrix((N, 1))  # column vector of zeros
-    bas[n - offset, 0] = 1  # 1 located at position n
-    bas = bas.tocsr()
+    data = np.array([1], dtype=complex)
+    ind = np.array([0], dtype=np.int32)
+    ptr = np.array([0]*((n - offset)+1)+[1]*(N-(n-offset)),dtype=np.int32)
 
-    return Qobj(bas)
+    return Qobj(fast_csr_matrix((data,ind,ptr), shape=(N,1)), isherm=False)
 
 
 def qutrit_basis():
@@ -121,11 +123,6 @@ def qutrit_basis():
     return np.array([basis(3, 0), basis(3, 1), basis(3, 2)], dtype=object)
 
 
-def _sqrt_factorial(n_vec):
-    # take the square root before multiplying
-    return np.array([np.prod(np.sqrt(np.arange(1, n + 1))) for n in n_vec])
-
-
 def coherent(N, alpha, offset=0, method='operator'):
     """Generates a coherent state with eigenvalue alpha.
 
@@ -185,11 +182,16 @@ def coherent(N, alpha, offset=0, method='operator'):
 
     elif method == "analytic" or offset > 0:
 
-        data = np.zeros([N, 1], dtype=complex)
-        n = arange(N) + offset
-        data[:, 0] = np.exp(-(abs(alpha) ** 2) / 2.0) * (alpha ** (n)) / \
-            _sqrt_factorial(n)
-        return Qobj(data)
+        sqrtn = np.sqrt(np.arange(offset, offset+N, dtype=complex))
+        sqrtn[0] = 1 # Get rid of divide by zero warning
+        data = alpha/sqrtn
+        if offset == 0:
+            data[0] = np.exp(-abs(alpha)**2 / 2.0)
+        else:
+            s = np.prod(np.sqrt(np.arange(1, offset + 1))) # sqrt factorial
+            data[0] = np.exp(-abs(alpha)**2 / 2.0) * alpha**(offset) / s
+        np.cumprod(data, out=sqrtn) # Reuse sqrtn array
+        return Qobj(sqrtn)
 
     else:
         raise TypeError(
@@ -458,11 +460,10 @@ shape = [3, 3], type = oper, isHerm = True
 # projection operator
 #
 def projection(N, n, m, offset=0):
-    """The projection operator that projects state |m> on state |n>: |n><m|.
+    """The projection operator that projects state :math:`|m>` on state :math:`|n>`.
 
     Parameters
     ----------
-
     N : int
         Number of basis states in Hilbert space.
 
@@ -477,6 +478,7 @@ def projection(N, n, m, offset=0):
     -------
     oper : qobj
          Requested projection operator.
+
     """
     ket1 = basis(N, n, offset=offset)
     ket2 = basis(N, m, offset=offset)
@@ -701,10 +703,10 @@ def state_number_enumerate(dims, excitations=None, state=None, idx=0):
 
         >>> for state in state_number_enumerate([2,2]):
         >>>     print(state)
-        [ 0.  0.]
-        [ 0.  1.]
-        [ 1.  0.]
-        [ 1.  1.]
+        [ 0  0 ]
+        [ 0  1 ]
+        [ 1  0 ]
+        [ 1  1 ]
 
     Parameters
     ----------
@@ -730,7 +732,7 @@ def state_number_enumerate(dims, excitations=None, state=None, idx=0):
     """
 
     if state is None:
-        state = np.zeros(len(dims))
+        state = np.zeros(len(dims), dtype=int)
 
     if excitations and sum(state[0:idx]) > excitations:
         pass
@@ -754,7 +756,7 @@ def state_number_index(dims, state):
     Example:
 
         >>> state_number_index([2, 2, 2], [1, 1, 0])
-        6.0
+        6
 
     Parameters
     ----------
@@ -766,12 +768,13 @@ def state_number_index(dims, state):
 
     Returns
     -------
-    idx : list
+    idx : int
         The index of the state given by `state` in standard enumeration
         ordering.
 
     """
-    return sum([state[i] * prod(dims[i + 1:]) for i, d in enumerate(dims)])
+    return int(
+        sum([state[i] * prod(dims[i + 1:]) for i, d in enumerate(dims)]))
 
 
 def state_index_number(dims, index):
@@ -1161,7 +1164,8 @@ def triplet_states():
     trip_states = []
     trip_states.append(tensor(basis(2, 1), basis(2, 1)))
     trip_states.append(
-        tensor(basis(2), basis(2, 1))+tensor(basis(2, 1), basis(2)))
+       (tensor(basis(2), basis(2, 1)) + tensor(basis(2, 1), basis(2))).unit()
+    )
     trip_states.append(tensor(basis(2), basis(2)))
     return trip_states
 
