@@ -38,8 +38,9 @@ __all__ = ['liouvillian', 'liouvillian_ref', 'lindblad_dissipator',
 import scipy.sparse as sp
 import numpy as np
 from qutip.qobj import Qobj
+from qutip.fastsparse import fast_csr_matrix, fast_identity
 from qutip.sparse import sp_reshape
-
+from qutip.cy.spmath import zcsr_kron
 
 def liouvillian(H, c_ops=[], data_only=False, chi=None):
     """Assembles the Liouvillian superoperator from a Hamiltonian
@@ -92,31 +93,33 @@ def liouvillian(H, c_ops=[], data_only=False, chi=None):
     sop_dims = [[op_dims[0], op_dims[0]], [op_dims[1], op_dims[1]]]
     sop_shape = [np.prod(op_dims), np.prod(op_dims)]
 
-    spI = sp.identity(op_shape[0], format='csr')
+    spI = fast_identity(op_shape[0])
 
     if H:
         if H.isoper:
-            data = -1j * (sp.kron(spI, H.data, format='csr')
-                          - sp.kron(H.data.T, spI, format='csr'))
+            Ht = H.data.T
+            data = -1j * zcsr_kron(spI, H.data)
+            data += 1j * zcsr_kron(Ht, spI)
         else:
             data = H.data
     else:
-        data = sp.csr_matrix((sop_shape[0], sop_shape[1]), dtype=complex)
+        data = fast_csr_matrix(shape=(sop_shape[0], sop_shape[1]))
 
     for idx, c_op in enumerate(c_ops):
         if c_op.issuper:
             data = data + c_op.data
         else:
-            cd = c_op.data.T.conj()
+            cd = c_op.data.H
             c = c_op.data
             if chi:
-                data = data + np.exp(1j * chi[idx]) * sp.kron(cd.T, c,
-                                                              format='csr')
+                data = data + np.exp(1j * chi[idx]) * \
+                                zcsr_kron(c.conj(), c)
             else:
-                data = data + sp.kron(cd.T, c, format='csr')
+                data = data + zcsr_kron(c.conj(), c)
             cdc = cd * c
-            data = data - 0.5 * sp.kron(spI, cdc, format='csr')
-            data = data - 0.5 * sp.kron(cdc.T, spI, format='csr')
+            cdct = cdc.T
+            data = data - 0.5 * zcsr_kron(spI, cdc)
+            data = data - 0.5 * zcsr_kron(cdct, spI)
 
     if data_only:
         return data
@@ -124,7 +127,6 @@ def liouvillian(H, c_ops=[], data_only=False, chi=None):
         L = Qobj()
         L.dims = sop_dims
         L.data = data
-        L.isherm = False
         L.superrep = 'super'
         return L
 
@@ -269,7 +271,8 @@ def spost(A):
 
     S = Qobj(isherm=A.isherm, superrep='super')
     S.dims = [[A.dims[0], A.dims[1]], [A.dims[0], A.dims[1]]]
-    S.data = sp.kron(A.data.T, sp.identity(np.prod(A.shape[0])), format='csr')
+    S.data = zcsr_kron(A.data.T, 
+                fast_identity(np.prod(A.shape[0])))
     return S
 
 
@@ -294,7 +297,7 @@ def spre(A):
 
     S = Qobj(isherm=A.isherm, superrep='super')
     S.dims = [[A.dims[0], A.dims[1]], [A.dims[0], A.dims[1]]]
-    S.data = sp.kron(sp.identity(np.prod(A.shape[1])), A.data, format='csr')
+    S.data = zcsr_kron(fast_identity(np.prod(A.shape[1])), A.data)
     return S
 
 
@@ -326,5 +329,5 @@ def sprepost(A, B):
 
     dims = [[_drop_projected_dims(A.dims[0]), _drop_projected_dims(B.dims[1])],
             [_drop_projected_dims(A.dims[1]), _drop_projected_dims(B.dims[0])]]
-    data = sp.kron(B.data.T, A.data, format='csr')
+    data = zcsr_kron(B.data.T, A.data)
     return Qobj(data, dims=dims, superrep='super')
