@@ -34,19 +34,57 @@
 import scipy.sparse as sp
 import scipy.linalg as la
 import numpy as np
-from numpy.testing import assert_equal, assert_, run_module_suite
+from numpy.testing import assert_equal, assert_, assert_almost_equal, run_module_suite
 
 from qutip.qobj import Qobj
 from qutip.random_objects import (rand_ket, rand_dm, rand_herm, rand_unitary,
-                                  rand_super)
+                                  rand_super, rand_super_bcsz, rand_dm_ginibre)
 from qutip.states import basis, fock_dm, ket2dm
-from qutip.operators import create, destroy, num, sigmax
-from qutip.superoperator import spre, spost, operator_to_vector
-from qutip.superop_reps import to_super
+from qutip.operators import create, destroy, num, sigmax, sigmay, qeye
+from qutip.superoperator import spre, spost, operator_to_vector, vector_to_operator
+from qutip.superop_reps import to_super, to_choi, to_chi
 from qutip.tensor import tensor, super_tensor, composite
 
 from operator import add, mul, truediv, sub
+from functools import partial
+from contextlib import contextmanager
 
+@contextmanager
+def expect_exception(ex_class):
+    """
+    Context manager that raises an AssertionError if its body
+    does not raise a particular class of exception.
+    """
+    ex_name = getattr(ex_class, '__name__', str(ex_class))
+
+    try:
+        yield
+    except ex_class:
+        pass
+    except Exception as other_ex:
+        raise AssertionError(
+            "Expected {}, but {} was raised.".format(
+                ex_name, getattr(type(other_ex), '__name__', str(type(other_ex)))
+            )
+        )
+    else:
+        raise AssertionError("Expected {}, but nothing was raised.".format(ex_name))
+
+def test_expect_exception():
+    with expect_exception(ValueError):
+        raise ValueError
+
+    with expect_exception(AssertionError):
+        with expect_exception(ValueError):
+            pass
+
+def assert_hermicity(oper, hermicity, msg=""):
+    # Check the cached isherm, if any exists.
+    assert_(oper.isherm == hermicity, msg)
+    # Force a reset of the cached value for isherm.
+    oper._isherm = None
+    # Force a recalculation of isherm.
+    assert_(oper.isherm == hermicity, msg)
 
 def test_QobjData():
     "Qobj data"
@@ -131,21 +169,27 @@ def test_QobjHerm():
 
     # test addition of two nonhermitian operators adding up to a hermitian one
     q_x = q_a + q_ad
-    assert_(q_x.isherm)  # isherm use the _isherm cache from q_a + q_ad
-    q_x._isherm = None   # reset _isherm cache
-    assert_(q_x.isherm)  # recalculate _isherm
+    assert_hermicity(q_x, True)
 
     # test addition of one hermitan and one nonhermitian operator
     q = q_x + q_a
-    assert_(not q.isherm)
-    q._isherm = None
-    assert_(not q.isherm)
+    assert_hermicity(q, False)
 
     # test addition of two hermitan operators
     q = q_x + q_x
-    assert_(q.isherm)
-    q._isherm = None
-    assert_(q.isherm)
+    assert_hermicity(q, True)
+
+    # Test multiplication of two Hermitian operators.
+    # This results in a skew-Hermitian operator, so
+    # we're checking here that __mul__ doesn't set wrong
+    # metadata.
+    q = sigmax() * sigmay()
+    assert_hermicity(q, False, "Expected iZ = X * Y to be skew-Hermitian.")
+    # Similarly, we need to check that -Z = X * iY is correctly
+    # identified as Hermitian.
+    q = sigmax() * (1j * sigmay())
+    assert_hermicity(q, True, "Expected -Z = X * iY to be Hermitian.")
+
 
 
 def test_QobjDimsShape():
@@ -174,6 +218,39 @@ def test_QobjDimsShape():
     assert_equal(q1.dims, [[2, 2], [2, 2]])
     assert_equal(q1.shape, [4, 4])
 
+def test_QobjMulNonsquareDims():
+    """
+    Qobj: multiplication w/ non-square qobj.dims
+
+    Checks for regression of #331.
+    """
+    data = np.array([[0, 1], [1, 0]])
+
+    q1 = Qobj(data)
+    q1.dims[0].append(1)
+    q2 = Qobj(data)
+
+    assert_equal((q1 * q2).dims, [[2, 1], [2]])
+    assert_equal((q2 * q1.dag()).dims, [[2], [2, 1]])
+
+    # Note that this is [[2], [2]] instead of [[2, 1], [2, 1]],
+    # as matching dimensions of 1 are implicitly partial traced out.
+    # (See #331.)
+    assert_equal((q1 * q2 * q1.dag()).dims, [[2], [2]])
+    
+    # Because of the above, we also need to check for extra indices
+    # that aren't of length 1.
+    q1 = Qobj([[ 1.+0.j,  0.+0.j],
+         [ 0.+0.j,  1.+0.j],
+         [ 0.+0.j,  1.+0.j],
+         [ 1.+0.j,  0.+0.j],
+         [ 0.+0.j,  0.-1.j],
+         [ 0.+1.j,  0.+0.j],
+         [ 1.+0.j,  0.+0.j],
+         [ 0.+0.j, -1.+0.j]
+     ], dims=[[4, 2], [2]])
+
+    assert_equal((q1 * q2 * q1.dag()).dims, [[4, 2], [4, 2]])
 
 def test_QobjAddition():
     "Qobj addition"
@@ -265,7 +342,7 @@ def test_QobjDivision():
     q = Qobj(data)
     randN = 10 * np.random.random()
     q = q / randN
-    assert_(np.all(q.data.todense() - np.matrix(data) / randN == 0))
+    assert_(np.allclose(q.data.todense(), np.matrix(data) / randN))
 
 
 def test_QobjPower():
@@ -374,6 +451,14 @@ def test_CheckMulType():
 
     assert_(opbra2.dag() == opket2)
 
+def test_Qobj_Spmv():
+    "Qobj mult ndarray right"
+    A = rand_herm(5)
+    b = rand_ket(5).full()
+    C = A*b
+    D = A.full().dot(b)
+    assert_(np.all((C-D)<1e-14))
+
 
 def test_QobjConjugate():
     "Qobj conjugate"
@@ -440,7 +525,7 @@ def test_QobjEigenStates():
 
 
 def test_QobjExpm():
-    "Qobj expm"
+    "Qobj expm (dense)"
     data = np.random.random(
         (15, 15)) + 1j * np.random.random((15, 15)) - (0.5 + 0.5j)
     A = Qobj(data)
@@ -449,25 +534,19 @@ def test_QobjExpm():
 
 
 def test_QobjExpmExplicitlySparse():
-    "Qobj expm (explicit sparse)"
+    "Qobj expm (sparse)"
     data = np.random.random(
         (15, 15)) + 1j * np.random.random((15, 15)) - (0.5 + 0.5j)
     A = Qobj(data)
     B = A.expm(method='sparse')
     assert_((B.data.todense() - np.matrix(la.expm(data)) < 1e-10).all())
-    B = A.expm(method='scipy-sparse')
-    assert_((B.data.todense() - np.matrix(la.expm(data)) < 1e-10).all())
 
 
-def test_QobjExpmExplicitDense():
-    "Qobj expm (explicit dense)"
-    data = np.random.random(
-        (15, 15)) + 1j * np.random.random((15, 15)) - (0.5 + 0.5j)
-    A = Qobj(data)
-    B = A.expm(method='dense')
-    assert_((B.data.todense() - np.matrix(la.expm(data)) < 1e-10).all())
-    B = A.expm(method='scipy-delse')
-    assert_((B.data.todense() - np.matrix(la.expm(data)) < 1e-10).all())
+def test_QobjExpmZeroOper():
+    "Qobj expm zero_oper (#493)"
+    A = Qobj(np.zeros((5,5), dtype=complex))
+    B = A.expm()
+    assert_(B == qeye(5))
 
 
 def test_Qobj_sqrtm():
@@ -507,35 +586,59 @@ def test_QobjNorm():
 
 def test_QobjPermute():
     "Qobj permute"
-    A = basis(5, 0)
+    A = basis(3, 0)
     B = basis(5, 4)
-    C = basis(5, 2)
+    C = basis(4, 2)
     psi = tensor(A, B, C)
     psi2 = psi.permute([2, 0, 1])
     assert_(psi2 == tensor(C, A, B))
+    
+    psi_bra = psi.dag()
+    psi2_bra = psi_bra.permute([2, 0, 1])
+    assert_(psi2_bra == tensor(C, A, B).dag())
 
-    A = fock_dm(5, 0)
+    A = fock_dm(3, 0)
     B = fock_dm(5, 4)
-    C = fock_dm(5, 2)
+    C = fock_dm(4, 2)
     rho = tensor(A, B, C)
     rho2 = rho.permute([2, 0, 1])
     assert_(rho2 == tensor(C, A, B))
 
     for ii in range(3):
-        A = rand_ket(5)
-        B = rand_ket(5)
+        A = rand_ket(3)
+        B = rand_ket(4)
         C = rand_ket(5)
         psi = tensor(A, B, C)
         psi2 = psi.permute([1, 0, 2])
         assert_(psi2 == tensor(B, A, C))
+        
+        psi_bra = psi.dag()
+        psi2_bra = psi_bra.permute([1, 0, 2])
+        assert_(psi2_bra == tensor(B, A, C).dag())
 
     for ii in range(3):
-        A = rand_dm(5)
-        B = rand_dm(5)
+        A = rand_dm(3)
+        B = rand_dm(4)
         C = rand_dm(5)
         rho = tensor(A, B, C)
         rho2 = rho.permute([1, 0, 2])
         assert_(rho2 == tensor(B, A, C))
+        
+        rho_vec = operator_to_vector(rho)
+        rho2_vec = rho_vec.permute([[1, 0, 2],[4,3,5]])
+        assert_(rho2_vec == operator_to_vector(tensor(B, A, C)))
+        
+        rho_vec_bra = operator_to_vector(rho).dag()
+        rho2_vec_bra = rho_vec_bra.permute([[1, 0, 2],[4,3,5]])
+        assert_(rho2_vec_bra == operator_to_vector(tensor(B, A, C)).dag())
+        
+    for ii in range(3):
+        super_dims = [3, 5, 4]
+        U = rand_unitary(np.prod(super_dims), density=0.02, dims=[super_dims, super_dims])
+        Unew = U.permute([2,1,0])
+        S_tens = to_super(U)
+        S_tens_new = to_super(Unew)
+        assert_(S_tens_new == S_tens.permute([[2,1,0],[5,4,3]]))
 
 
 def test_KetType():
@@ -607,6 +710,24 @@ def test_SuperType():
     assert_equal(sop.issuper, True)
 
 
+def test_dag_preserves_superrep():
+    """
+    Checks that dag() preserves superrep.
+    """
+
+    def case(qobj):
+        orig_superrep = qobj.superrep
+        assert_equal(qobj.dag().superrep, orig_superrep)
+
+    for dim in (2, 4, 8):
+        qobj = rand_super_bcsz(dim)
+        yield case, to_super(qobj)
+        # These two shouldn't even do anything, since qobj
+        # is Hermicity-preserving.
+        yield case, to_choi(qobj)
+        yield case, to_chi(qobj)
+
+
 def test_arithmetic_preserves_superrep():
     """
     Checks that binary ops preserve 'superrep'.
@@ -725,5 +846,152 @@ def test_composite_vec():
     assert_(composite(k1, r4) == super_tensor(r1, r4))
     assert_(composite(r3, k2) == super_tensor(r3, r2))
 
+# TODO: move out to a more appropriate module.
+
+def has_description(case):
+    """
+    Decorates a test case such that it takes an argument describing
+    that case, such that failure logs are usefully formatted.
+    """
+    # See https://code.google.com/archive/p/python-nose/issues/244#c1
+    # for why this works.
+
+    def make_case(description):
+        partial_case = partial(case)
+        partial_case.description = description
+        return partial_case
+    return make_case
+
+def test_trunc_neg():
+    """
+    Test Qobj: Checks trunc_neg in several different cases.
+    """
+
+    @has_description
+    def case(qobj, method, expected=None):
+        pos_qobj = qobj.trunc_neg(method=method)
+        assert(all([energy > -1e-8 for energy in pos_qobj.eigenenergies()]))
+        assert_almost_equal(pos_qobj.tr(), 1)
+
+        if expected is not None:
+            assert_almost_equal(pos_qobj.data.todense(), expected.data.todense())
+
+    for method in ('clip', 'sgs'):
+        # Make sure that it works for operators that are already positive.
+        yield case("Test Qobj: trunc_neg works for positive opers."), \
+            rand_dm(5), method
+        # Make sure that it works for a diagonal matrix.
+        yield case("Test Qobj: trunc_neg works for diagonal opers."), \
+            Qobj(np.diag([1.1, -0.1])), method, Qobj(np.diag([1.0, 0.0]))
+        # Make sure that it works for a non-diagonal matrix.
+        U = rand_unitary(3)
+        yield case("Test Qobj: trunc_neg works for non-diagonal opers."), \
+            U * Qobj(np.diag([1.1, 0, -0.1])) * U.dag(), \
+            method, \
+            U * Qobj(np.diag([1.0, 0.0, 0.0])) * U.dag()
+
+    # Check the test case in SGS.
+    yield (
+        case("Test Qobj: trunc_neg works for SGS known-good test case."),
+        Qobj(np.diag([3. / 5, 1. / 2, 7. / 20, 1. / 10, -11. / 20])), 'sgs',
+        Qobj(np.diag([9. / 20, 7. / 20, 1. / 5, 0, 0]))
+    )
+    
+
+def test_cosm():
+    """
+    Test Qobj: cosm
+    """
+    A = rand_herm(5)
+
+    B = A.cosm().full()
+
+    C = la.cosm(A.full())
+
+    assert_(np.all((B-C)< 1e-14))
+
+
+def test_sinm():
+    """
+    Test Qobj: sinm
+    """
+    A = rand_herm(5)
+
+    B = A.sinm().full()
+
+    C = la.sinm(A.full())
+
+    assert_(np.all((B-C)< 1e-14))
+
+
+
+def test_dual_channel():
+    """
+    Qobj: dual_chan() preserves inner products with arbitrary density ops.
+    """
+
+    def case(S, n_trials=50):        
+        S = to_super(S)
+        left_dims, right_dims = S.dims
+        
+        # Assume for the purposes of the test that S maps square operators to square operators.
+        in_dim = np.prod(right_dims[0])
+        out_dim = np.prod(left_dims[0])
+        
+        S_dual = to_super(S.dual_chan())
+        
+        primals = []
+        duals = []
+    
+        for idx_trial in range(n_trials):
+            X = rand_dm_ginibre(out_dim)
+            X.dims = left_dims
+            X = operator_to_vector(X)
+            Y = rand_dm_ginibre(in_dim)
+            Y.dims = right_dims
+            Y = operator_to_vector(Y)
+
+            primals.append((X.dag() * S * Y)[0, 0])
+            duals.append((X.dag() * S_dual.dag() * Y)[0, 0])
+    
+        np.testing.assert_array_almost_equal(primals, duals)
+
+    for subdims in [
+        [2],
+        [2, 2],
+        [2, 3],
+        [3, 5, 2]
+    ]:
+        dim = np.prod(subdims)
+        S = rand_super_bcsz(dim)
+        S.dims = [[subdims, subdims], [subdims, subdims]]
+        yield case, S
+
+
+def test_call():
+    # Make test objects.
+    psi = rand_ket(3)
+    rho = rand_dm_ginibre(3)
+    U = rand_unitary(3)
+    S = rand_super_bcsz(3)
+
+    # Case 0: oper(ket).
+    assert U(psi) == U * psi
+
+    # Case 1: oper(oper). Should raise TypeError.
+    with expect_exception(TypeError):
+        U(rho)
+
+    # Case 2: super(ket).
+    assert S(psi) == vector_to_operator(S * operator_to_vector(ket2dm(psi)))
+
+    # Case 3: super(oper).
+    assert S(rho) == vector_to_operator(S * operator_to_vector(rho))
+
+    # Case 4: super(super). Should raise TypeError.
+    with expect_exception(TypeError):
+        S(S)
+
+
 if __name__ == "__main__":
     run_module_suite()
