@@ -49,9 +49,11 @@ from qutip.states import ket2dm
 from qutip.parallel import parfor
 from qutip.utilities import clebsch
 from scipy.misc import factorial
+from qutip.cy.sparse_utils import _csr_get_diag
 
 
-def wigner(psi, xvec, yvec, method='iterative', g=sqrt(2), parfor=False):
+def wigner(psi, xvec, yvec, method='clenshaw', g=sqrt(2), 
+            sparse=False, parfor=False):
     """Wigner function for a state vector or density matrix at points
     `xvec + i * yvec`.
 
@@ -71,18 +73,23 @@ def wigner(psi, xvec, yvec, method='iterative', g=sqrt(2), parfor=False):
     g : float
         Scaling factor for `a = 0.5 * g * (x + iy)`, default `g = sqrt(2)`.
 
-    method : string {'iterative', 'laguerre', 'fft'}
-        Select method 'iterative', 'laguerre', or 'fft', where 'iterative' uses
-        an iterative method to evaluate the Wigner functions for density
+    method : string {'clenshaw', 'iterative', 'laguerre', 'fft'}
+        Select method 'clenshaw' 'iterative', 'laguerre', or 'fft', where 'clenshaw' 
+        and 'iterative' use an iterative method to evaluate the Wigner functions for density
         matrices :math:`|m><n|`, while 'laguerre' uses the Laguerre polynomials
         in scipy for the same task. The 'fft' method evaluates the Fourier
         transform of the density matrix. The 'iterative' method is default, and
         in general recommended, but the 'laguerre' method is more efficient for
         very sparse density matrices (e.g., superpositions of Fock states in a
-        large Hilbert space). The 'fft' method is the preferred method for
+        large Hilbert space). The 'clenshaw' method is the preferred method for
         dealing with density matrices that have a large number of excitations
-        (>~50).
+        (>~50). 'clenshaw' is a fast and numerically stable method.
 
+    sparse : bool {False, True}
+        Tells the default solver whether or not to keep the input density
+        matrix in sparse format.  As the dimensions of the density matrix
+        grow, setthing this flag can result in increased performance.
+    
     parfor : bool {False, True}
         Flag for calculating the Laguerre polynomial based Wigner function
         method='laguerre' in parallel using the parfor function.
@@ -128,6 +135,9 @@ def wigner(psi, xvec, yvec, method='iterative', g=sqrt(2), parfor=False):
 
     elif method == 'laguerre':
         return _wigner_laguerre(rho, xvec, yvec, g, parfor)
+        
+    elif method == 'clenshaw':
+        return _wigner_clenshaw(rho, xvec, yvec, g, sparse=sparse)
 
     else:
         raise TypeError(
@@ -287,18 +297,18 @@ def _wigner_fft(psi, xvec):
     Evaluates the Fourier transformation of a given state vector.
     Returns the corresponding density matrix and range
     """
-    n = 2 * len(psi.T)
+    n = 2*len(psi.T)
     r1 = np.concatenate((np.array([[0]]),
                         np.fliplr(psi.conj()),
-                        np.zeros((1, n / 2 - 1))), axis=1)
+                        np.zeros((1, n//2 - 1))), axis=1)
     r2 = np.concatenate((np.array([[0]]), psi,
-                        np.zeros((1, n / 2 - 1))), axis=1)
-    w = la.toeplitz(np.zeros((n / 2, 1)), r1) * \
-        np.flipud(la.toeplitz(np.zeros((n / 2, 1)), r2))
-    w = np.concatenate((w[:, n / 2:n], w[:, 0:n / 2]), axis=1)
+                        np.zeros((1, n//2 - 1))), axis=1)
+    w = la.toeplitz(np.zeros((n//2, 1)), r1) * \
+        np.flipud(la.toeplitz(np.zeros((n//2, 1)), r2))
+    w = np.concatenate((w[:, n//2:n], w[:, 0:n//2]), axis=1)
     w = ft.fft(w)
-    w = np.real(np.concatenate((w[:, 3 * n / 4:n + 1], w[:, 0:n / 4]), axis=1))
-    p = np.arange(-n / 4, n / 4) * np.pi / (n * (xvec[1] - xvec[0]))
+    w = np.real(np.concatenate((w[:, 3*n//4:n+1], w[:, 0:n//4]), axis=1))
+    p = np.arange(-n/4, n/4)*np.pi / (n*(xvec[1] - xvec[0]))
     w = w / (p[1] - p[0]) / n
     return w, p
 
@@ -322,6 +332,77 @@ def _osc_eigen(N, pnts):
         return A
 
 
+def _wigner_clenshaw(rho, xvec, yvec, g=sqrt(2), sparse=False):
+    """
+    Using Clenshaw summation - numerically stable and efficient
+    iterative algorithm to evaluate polynomial series.
+    
+    The Wigner function is calculated as
+    :math:`W = e^(-0.5*x^2)/pi * \sum_{L} c_L (2x)^L / sqrt(L!)` where 
+    :math:`c_L = \sum_n \\rho_{n,L+n} LL_n^L` where
+    :math:`LL_n^L = (-1)^n sqrt(L!n!/(L+n)!) LaguerreL[n,L,x]`
+    
+    """
+
+    M = np.prod(rho.shape[0])
+    X,Y = np.meshgrid(xvec, yvec)
+    #A = 0.5 * g * (X + 1.0j * Y)
+    A2 = g * (X + 1.0j * Y) #this is A2 = 2*A
+    
+    B = np.abs(A2)
+    B *= B
+    w0 = (2*rho.data[0,-1])*np.ones_like(A2)
+    L = M-1
+    #calculation of \sum_{L} c_L (2x)^L / sqrt(L!)
+    #using Horner's method
+    if not sparse:
+        rho = rho.full() * (2*np.ones((M,M)) - np.diag(np.ones(M)))
+        while L > 0:
+            L -= 1
+            #here c_L = _wig_laguerre_val(L, B, np.diag(rho, L))
+            w0 = _wig_laguerre_val(L, B, np.diag(rho, L)) + w0 * A2 * (L+1)**-0.5
+    else:
+        while L > 0:
+            L -= 1
+            diag = _csr_get_diag(rho.data.data,rho.data.indices,
+                                rho.data.indptr,L)
+            if L != 0:
+                diag *= 2
+            #here c_L = _wig_laguerre_val(L, B, np.diag(rho, L))
+            w0 = _wig_laguerre_val(L, B, diag) + w0 * A2 * (L+1)**-0.5
+        
+    return w0.real * np.exp(-B*0.5) * (g*g*0.5 / pi)
+
+
+def _wig_laguerre_val(L, x, c):
+    """
+    this is evaluation of polynomial series inspired by hermval from numpy.    
+    Returns polynomial series
+    \sum_n b_n LL_n^L,
+    where
+    LL_n^L = (-1)^n sqrt(L!n!/(L+n)!) LaguerreL[n,L,x]    
+    The evaluation uses Clenshaw recursion
+    """
+
+    if len(c) == 1:
+        y0 = c[0]
+        y1 = 0
+    elif len(c) == 2:
+        y0 = c[0]
+        y1 = c[1]
+    else:
+        k = len(c)
+        y0 = c[-2]
+        y1 = c[-1]
+        for i in range(3, len(c) + 1):
+            k -= 1
+            y0,    y1 = c[-i] - y1 * (float((k - 1)*(L + k - 1))/((L+k)*k))**0.5, \
+            y0 - y1 * ((L + 2*k -1) - x) * ((L+k)*k)**-0.5
+            
+    return y0 - y1 * ((L + 1) - x) * (L + 1)**-0.5
+    
+    
+
 # -----------------------------------------------------------------------------
 # Q FUNCTION
 #
@@ -404,19 +485,15 @@ def spin_q_function(rho, theta, phi):
 
     Parameters
     ----------
-
     state : qobj
         A state vector or density matrix for a spin-j quantum system.
-
     theta : array_like
         theta-coordinates at which to calculate the Q function.
-
     phi : array_like
         phi-coordinates at which to calculate the Q function.
 
     Returns
     -------
-
     Q, THETA, PHI : 2d-array
         Values representing the spin Q function at the values specified
         by THETA and PHI.
@@ -430,18 +507,18 @@ def spin_q_function(rho, theta, phi):
         rho = ket2dm(rho)
 
     J = rho.shape[0]
-    j = (J - 1) / 2
+    j = int((J - 1) / 2)
 
     THETA, PHI = meshgrid(theta, phi)
 
     Q = np.zeros_like(THETA, dtype=complex)
 
-    for m1 in arange(-j, j+1):
+    for m1 in range(-j, j+1):
 
         Q += binom(2*j, j+m1) * cos(THETA/2) ** (2*(j-m1)) * sin(THETA/2) ** (2*(j+m1)) * \
             rho.data[int(j-m1), int(j-m1)]
 
-        for m2 in arange(m1+1, j+1):
+        for m2 in range(m1+1, j+1):
 
             Q += (sqrt(binom(2*j, j+m1)) * sqrt(binom(2*j, j+m2)) *
                   cos(THETA/2) ** (2*j-m1-m2) * sin(THETA/2) ** (2*j+m1+m2)) * \
@@ -454,8 +531,8 @@ def spin_q_function(rho, theta, phi):
 def _rho_kq(rho, j, k, q):
     v = 0j
 
-    for m1 in arange(-j, j+1):
-        for m2 in arange(-j, j+1):
+    for m1 in range(-j, j+1):
+        for m2 in range(-j, j+1):
             v += (-1)**(j - m1 - q) * clebsch(j, j, k, m1, -m2,
                                               q) * rho.data[m1 + j, m2 + j]
 
@@ -463,30 +540,26 @@ def _rho_kq(rho, j, k, q):
 
 
 def spin_wigner(rho, theta, phi):
-    """Wigner function for spins.
+    """Wigner function for spins on the Bloch sphere.
 
     Parameters
     ----------
-
     state : qobj
         A state vector or density matrix for a spin-j quantum system.
-
     theta : array_like
         theta-coordinates at which to calculate the Q function.
-
     phi : array_like
         phi-coordinates at which to calculate the Q function.
 
     Returns
     -------
-
     W, THETA, PHI : 2d-array
         Values representing the spin Wigner function at the values specified
         by THETA and PHI.
 
-    .. note::
-
-        Experimental.
+    Notes
+    -----
+    Experimental.
 
     """
 
@@ -497,7 +570,7 @@ def spin_wigner(rho, theta, phi):
         rho = ket2dm(rho)
 
     J = rho.shape[0]
-    j = (J - 1) / 2
+    j = int((J - 1) / 2)
 
     THETA, PHI = meshgrid(theta, phi)
 
